/*
This is a part of the One LiteStep module source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
erik@alphafish.com
www.alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "one.h"

char g_szClosed[MAX_BANGARGS];
char g_szOpen[MAX_BANGARGS];
char g_szActive[MAX_BANGARGS];

int initModuleEx(HWND /* hParent */, HINSTANCE /* hInstance */, LPCSTR /* szPath */)
{
	GetRCLine("OneClosedCmd", g_szClosed, MAX_BANGARGS, ".none");
	GetRCLine("OneOpenCmd", g_szOpen, MAX_BANGARGS, ".focus");
	GetRCLine("OneActiveCmd", g_szActive, MAX_BANGARGS, "");
	
	AddBangCommand("!One", bangOne);

	return 0;
}

void quitModule(HINSTANCE /* hInstance */)
{
	RemoveBangCommand("!One");
}

void ProcessCommand(HWND hWnd, HWND hCaller, LPCSTR pszArgument)
{
	if(!stricmp(pszArgument, ".none")) {
		return;
	}
	else if(!stricmp(pszArgument, ".focus")) {
		SwitchToThisWindow(hWnd, TRUE);
	}
	else if(!stricmp(pszArgument, ".minimize")) {
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MINIMIZE, NULL);
	}
	else if(!stricmp(pszArgument, ".maximize")) {
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MAXIMIZE, NULL);
	}
	else if(!stricmp(pszArgument, ".restore")) {
		SendMessage(hWnd, WM_SYSCOMMAND, SC_RESTORE, NULL);
	}
	else if(!stricmp(pszArgument, ".close")) {
		SendMessage(hWnd, WM_SYSCOMMAND, SC_CLOSE, NULL);
	}
	else if(!stricmp(pszArgument, ".show")) {
		ShowWindow(hWnd,SW_SHOW);
	}
	else if(!stricmp(pszArgument, ".hide")) {
		ShowWindow(hWnd,SW_HIDE);
	}
	else {
		LSExecute(hCaller, pszArgument, NULL);
	}
}

void bangOne(HWND hCaller, LPCSTR pszArgs)
{
	HWND hWnd = NULL;

	char szFirst[MAX_BANGARGS];
	char szSecond[MAX_BANGARGS];
	char szThird[MAX_BANGARGS];
	char szFourth[MAX_BANGARGS];
	char szFifth[MAX_BANGARGS];

	char* pszClass = szFirst;
	char* pszTitle = NULL;
	char* pszClosed = NULL;
	char* pszOpen = NULL;
	char* pszActive = NULL;

	LPSTR lpszTokens[] = { szFirst, szSecond, szThird, szFourth, szFifth };
	
	CommandTokenize(pszArgs, lpszTokens, 5, NULL);

	
	if(!StrNCmpI(pszClass, "hwnd:", 5)) {
		hWnd = (HWND)atol(pszClass + 5);
		if(IsWindow(hWnd) == FALSE) {
			hWnd = NULL;
		}
		pszClosed = szSecond;
		pszOpen = szThird;
		pszActive = szFourth;
	}
	else {
		pszTitle = szSecond;
		pszClosed = szThird;
		pszOpen = szFourth;
		pszActive = szFifth;
		hWnd = FindWindow(strlen(pszClass) ? pszClass : NULL, strlen(pszTitle) ? pszTitle : NULL);
	}

	if(strlen(pszClosed) == 0) {
		StringCchCopy(pszClosed, MAX_BANGARGS, g_szClosed);
	}

	if(strlen(pszOpen) == 0) {
		StringCchCopy(pszOpen, MAX_BANGARGS, g_szOpen);
	}

	if(strlen(pszActive) == 0) {
		StringCchCopy(pszActive, MAX_BANGARGS, strlen(g_szActive) ? g_szActive : pszOpen);
	}

	if(!hWnd) {
		LSExecute(hCaller, pszClosed, SW_SHOW);
	}
	else {
		if(hWnd == GetForegroundWindow()) {
			ProcessCommand(hWnd, hCaller, pszActive);
		}
		else {
			ProcessCommand(hWnd, hCaller, pszOpen);
		}
	}
}
